#!/bin/bash

# This script requires root privileges to run.
# The script will iterate through the `/etc/passwd` file to retrieve each user's UID, GID, and home directory.
# If the home directory does not exist, it will create the home directory and copy files from the /etc/skel directory to the home directory.
# If the home directory exists, it will check if the ownership matches the UID and GID; if not, it will change the ownership.

# Function to recursively change the owner of a directory and its contents
chown_home_dir() {
    local home_dir="$1"
    local target_uid="$2"
    local target_gid="$3"

    # Get the current UID and GID of the directory
    local current_uid_gid
    current_uid_gid=$(stat -c '%u:%g' "$home_dir")
    IFS=: read -r current_uid current_gid <<< "$current_uid_gid"

    # Check if a change is needed
    if [[ "$current_uid" -ne "$target_uid" || "$current_gid" -ne "$target_gid" ]]; then
        echo "Changing ownership of $home_dir from UID $current_uid:GID $current_gid to UID $target_uid:GID $target_gid"
        chown -R "$target_uid:$target_gid" "$home_dir"
    else
        echo "No need to change ownership for $home_dir (UID $current_uid:GID $current_gid)"
    fi
}

main() {
    local uid_min=1000
    local uid_max=60000
    awk -F: "\$3 >= $uid_min && \$3 < $uid_max {OFS=\":\"; print \$3,\$4,\$6}" /etc/passwd | while IFS=: read -r uid gid home_dir; do
        # skip if home_dir is empty
        [ -z "$home_dir" ] && continue
        if [ -e "$home_dir" ]; then
            echo "Processing home directory: $home_dir"
            chown_home_dir "$home_dir" "$uid" "$gid"
        else
            echo "Home directory $home_dir does not exist"
            # make sure the home directory exists
            mkdir -p "$home_dir"
            cp -a /etc/skel/. "$home_dir"
            chown -R "$uid:$gid" "$home_dir"
        fi
    done
}

# Call the main function
main
